<?php

namespace Newage\Vote\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Newage\Vote\Http\Requests\VoteItemRequest;
use Newage\Vote\Interfaces\VoteItemRepositoryInterface;
use Newage\Vote\Models\VoteItem;

/**
 * Resource controller class for vote_item.
 */
class VoteItemResourceController extends BaseController
{

    /**
     * Initialize vote_item resource controller.
     *
     * @param type VoteItemRepositoryInterface $vote_item
     *
     * @return null
     */
    public function __construct(VoteItemRepositoryInterface $vote_item)
    {
        parent::__construct();
        $this->repository = $vote_item;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Newage\Vote\Repositories\Criteria\VoteItemResourceCriteria::class);
    }

    /**
     * Display a list of vote_item.
     *
     * @return Response
     */
    public function index(VoteItemRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Newage\Vote\Repositories\Presenter\VoteItemPresenter::class)
                ->$function();
        }

        $vote_items = $this->repository->paginate();

        return $this->response->title(trans('vote::vote_item.names'))
            ->view('vote::vote_item.index', true)
            ->data(compact('vote_items'))
            ->output();
    }

    /**
     * Display vote_item.
     *
     * @param Request $request
     * @param Model   $vote_item
     *
     * @return Response
     */
    public function show(VoteItemRequest $request, VoteItem $vote_item)
    {

        if ($vote_item->exists) {
            $view = 'vote::vote_item.show';
        } else {
            $view = 'vote::vote_item.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('vote::vote_item.name'))
            ->data(compact('vote_item'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new vote_item.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(VoteItemRequest $request)
    {

        $vote_item = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('vote::vote_item.name')) 
            ->view('vote::vote_item.create', true) 
            ->data(compact('vote_item'))
            ->output();
    }

    /**
     * Create new vote_item.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(VoteItemRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $vote_item                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('vote::vote_item.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('vote/vote_item/' . $vote_item->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/vote/vote_item'))
                ->redirect();
        }

    }

    /**
     * Show vote_item for editing.
     *
     * @param Request $request
     * @param Model   $vote_item
     *
     * @return Response
     */
    public function edit(VoteItemRequest $request, VoteItem $vote_item)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('vote::vote_item.name'))
            ->view('vote::vote_item.edit', true)
            ->data(compact('vote_item'))
            ->output();
    }

    /**
     * Update the vote_item.
     *
     * @param Request $request
     * @param Model   $vote_item
     *
     * @return Response
     */
    public function update(VoteItemRequest $request, VoteItem $vote_item)
    {
        try {
            $attributes = $request->all();

            $vote_item->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('vote::vote_item.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('vote/vote_item/' . $vote_item->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('vote/vote_item/' . $vote_item->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the vote_item.
     *
     * @param Model   $vote_item
     *
     * @return Response
     */
    public function destroy(VoteItemRequest $request, VoteItem $vote_item)
    {
        try {

            $vote_item->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('vote::vote_item.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('vote/vote_item/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('vote/vote_item/' . $vote_item->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple vote_item.
     *
     * @param Model   $vote_item
     *
     * @return Response
     */
    public function delete(VoteItemRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('vote::vote_item.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('vote/vote_item'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/vote/vote_item'))
                ->redirect();
        }

    }

    /**
     * Restore deleted vote_items.
     *
     * @param Model   $vote_item
     *
     * @return Response
     */
    public function restore(VoteItemRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('vote::vote_item.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/vote/vote_item'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/vote/vote_item/'))
                ->redirect();
        }

    }

}
